----------------------------------------------------------------------------------
-- ECE 3401 Spring 2024: Programming Assignment 1's First Test Bench
-- University of Connecticut
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
 
entity testbench1 is
end testbench1;
 
architecture Behavior OF testbench1 IS 
 component pa1
    port(clk, reset : in  std_logic;
         m1_sel, m2_sel : in  unsigned(1 downto 0);
         r1_en, r2_en : in std_logic;
         alu1_sel, alu2_sel : in  unsigned(1 downto 0);
         A, B, C, D : in  unsigned(15 downto 0);
         Z1, Z2 : out  unsigned(31 downto 0));
    end component;

   -- Clock period definitions
   constant clk_period : time := 10 ns;
   
   -- Inputs 
   signal a, b, c, d : unsigned(15 downto 0) := (others => '0');
   signal clk, reset, r1_en, r2_en : std_logic := '0';
   signal alu1_sel, alu2_sel : unsigned(1 downto 0) := (others => '0');
   signal m1_sel, m2_sel : unsigned(1 downto 0) := (others => '0');

   -- Outputs
   signal Z1, Z2 : unsigned(31 downto 0);
 
begin
   
    -- Instantiate the Unit Under Test (UUT)
   uut: pa1 port map (
          clk => clk,
          reset => reset,
          m1_sel => m1_sel,
          m2_sel => m2_sel,
          r1_en => r1_en,
          r2_en => r2_en,
          alu1_sel => alu1_sel,
          alu2_sel => alu2_sel,
          A => a,
          B => b,
          C => c,
          D => d,
          Z1 => Z1,
          Z2 => Z2);

   -- Clock process definitions
   clk_process: process
   begin
        clk <= '0';
        wait for clk_period/2;
        clk <= '1';
        wait for clk_period/2;
   end process;

     -- Stimulus process
   stim_proc: process
   begin
   
        --TODO: Read through the following code and ask questions if you have any
        
        --Reset the registers, then wait a clock period. Make sure 
        --to disable after and wait!
        reset <= '1';
        wait for clk_period;
        reset <= '0';
        wait for clk_period;
	
        --Here, we will walk through the calculation of n(n+1)/2 for parallel ALUs
        
        --First we need to enable both R1 and R2, so that way their inputs D propogate 
        --to the output Q on the rising clock edge
        r1_en <= '1'; --Enable R1
        r2_en <= '1'; --Enable R2
            
        --Next, we need to set the inputs a, b, c, and d. 
        --ALU1 finds f(3), so a = 3 and b = 1.
        --ALU2 finds f(5), so c = 5 and b = 1. 
        --Note: to_unsigned(x, len) turns value x into len-bit unsigned vector
        a <= to_unsigned(3, 16); 
        b <= to_unsigned(1, 16);
        c <= to_unsigned(5, 16);
        d <= to_unsigned(1, 16);
            
        --Next, we will set both ALU select signals as addition. 
        --This will give us n+1 for both ALUs
        alu1_sel <= "00"; --Set ALU1 to add
        alu2_sel <= "00"; --Set ALU1 to add
            
        --Finally, we should set up all of the MUX signals. Since we are using 
        --B and D in the computation, set them to "00"
        m1_sel <= "00"; --M1 select input B
        m2_sel <= "00"; --M2 select input D
              
        --We are done setting up the signals we wanted; let's wait for a clock period
        --to let all of the registers update.
        wait for clk_period;
        --Z1 = (3+1) and Z2 = (5+1). Great!
            
        --Next, we will calculate n(n+1) in both ALUs
            
        --Notice we already have a = 3 and b = 5. Since we did this previously
        --without changing those values, we do not need to do anything with those. 
            
        --Next we should set the ALUs to compute multiplication 
        alu1_sel <= "10"; --Set ALU1 to multiply
        alu2_sel <= "10"; --Set ALU2 to multiply
            
        --We can get n(n+1) by using the feedback from prev addition 
        --so set up the MUXes accordingly
        m1_sel <= "01"; 
        m2_sel <= "01"; 
            
        --Done setting up signals; let a clock cycle pass
        wait for clk_period;
        --Now we have Z1 = (3)(4) and Z2 = (5)(6)
        --Just need to take the output of each ALU and shift right by 1 to div by 2
        
        --Set up ALUs for right shift
        alu1_sel <= "11"; 
        alu2_sel <= "11"; 
            
        --MUX's are already set to self feedback - no change needed 
        wait for clk_period;
            
        --Now Z1 = (3)(4)/2 and Z2=(5)(6)/2, and we are done!
            
        --Hold the result for a few full periods
        r1_en <= '0';
        r2_en <= '0'; 
        wait for 5*clk_period; 
    end process;
end Behavior;
